@@ -14,14 +14,21 @@
 #include <string.h>
 #include <time.h>
 #include <ctype.h>
+#include <stdio.h>
+#include <stdlib.h>
 
-#include "gsmcal.h"
+#include <gammu-calendar.h>
+#include <gammu-unicode.h>
+
+#include "../misc/misc.h"
 #include "gsmmisc.h"
+#include "gsmcal.h"
 #include "../misc/coding/coding.h"
 
 void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday, GSM_CalendarEntry *entry)
 {
-	int		i,start=-1,frequency=-1,dow=-1,day=-1,month=-1,end=-1,Recurrance = 0, Repeat=0;
+	unsigned int i;
+	int start=-1,frequency=-1,dow=-1,day=-1,month=-1,end=-1,Recurrance = 0, Repeat=0, j;
 	char 		Buf[20];
 	GSM_DateTime	DT;
 	time_t		t_time1,t_time2;
@@ -29,19 +36,19 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 	rec[0] = 0;
 	rec[1] = 0;
 
-	for (i=0;i<entry->EntriesNum;i++) {
-		if (entry->Entries[i].EntryType == CAL_START_DATETIME)   start 		= i;
-		if (entry->Entries[i].EntryType == CAL_REPEAT_FREQUENCY) frequency 	= i;
-		if (entry->Entries[i].EntryType == CAL_REPEAT_DAYOFWEEK) dow 		= i;
-		if (entry->Entries[i].EntryType == CAL_REPEAT_DAY)       day 		= i;
-		if (entry->Entries[i].EntryType == CAL_REPEAT_MONTH)     month 		= i;
-		if (entry->Entries[i].EntryType == CAL_REPEAT_STOPDATE)  end 		= i;
+	for (j = 0; j < entry->EntriesNum; j++) {
+		if (entry->Entries[j].EntryType == CAL_START_DATETIME)   start 		= j;
+		if (entry->Entries[j].EntryType == CAL_REPEAT_FREQUENCY) frequency 	= j;
+		if (entry->Entries[j].EntryType == CAL_REPEAT_DAYOFWEEK) dow 		= j;
+		if (entry->Entries[j].EntryType == CAL_REPEAT_DAY)       day 		= j;
+		if (entry->Entries[j].EntryType == CAL_REPEAT_MONTH)     month 		= j;
+		if (entry->Entries[j].EntryType == CAL_REPEAT_STOPDATE)  end 		= j;
 	}
 	if (start == -1) return;
 
 	if (frequency != -1 && dow == -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1) {
-			//each day
+			/* each day */
 			Recurrance = 24;
 		}
 	}
@@ -60,21 +67,21 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 	if (frequency != -1 && dow != -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1 &&
 		    entry->Entries[dow].Number == i) {
-			//one week
+			/* one week */
 			Recurrance = 24*7;
 		}
 	}
 	if (frequency != -1 && dow != -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 2 &&
 		    entry->Entries[dow].Number == i) {
-			//two weeks
+			/* two weeks */
 			Recurrance = 24*14;
 		}
 	}
 	if (frequency != -1 && dow == -1 && day != -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1 &&
 		    entry->Entries[day].Number == entry->Entries[start].Date.Day) {
-			//month
+			/* month */
 			Recurrance = 0xffff-1;
 		}
 	}
@@ -82,7 +89,7 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 		if (entry->Entries[frequency].Number == 1 &&
 		    entry->Entries[day].Number == entry->Entries[start].Date.Day &&
 		    entry->Entries[month].Number == entry->Entries[start].Date.Month) {
-			//year
+			/* year */
 			Recurrance = 0xffff;
 		}
 	}
@@ -235,7 +242,7 @@ void GSM_GetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 		entry->Entries[entry->EntriesNum-1].Date.Second);
 }
 
-bool IsCalendarNoteFromThePast(GSM_CalendarEntry *note)
+bool GSM_IsCalendarNoteFromThePast(GSM_CalendarEntry *note)
 {
 	bool 		Past = true;
 	int		i,End=-1;
@@ -377,9 +384,9 @@ GSM_Error GSM_Translate_Category (GSM_CatTranslation direction, char *string, GS
 	case TRANSL_TO_GSM:
 		if (strstr(string,"MEETING")) 			*Type = GSM_CAL_MEETING;
 		else if (strstr(string,"REMINDER")) 		*Type = GSM_CAL_REMINDER;
-		else if (strstr(string,"DATE"))	 		*Type = GSM_CAL_REMINDER; //SE
-		else if (strstr(string,"TRAVEL"))	 	*Type = GSM_CAL_TRAVEL;   //SE
-		else if (strstr(string,"VACATION"))	 	*Type = GSM_CAL_VACATION; //SE
+		else if (strstr(string,"DATE"))	 		*Type = GSM_CAL_REMINDER; /* SE */
+		else if (strstr(string,"TRAVEL"))	 	*Type = GSM_CAL_TRAVEL;   /* SE */
+		else if (strstr(string,"VACATION"))	 	*Type = GSM_CAL_VACATION; /* SE */
 		else if (strstr(string,"MISCELLANEOUS"))	*Type = GSM_CAL_MEMO;
 		else if (strstr(string,"PHONE CALL")) 		*Type = GSM_CAL_CALL;
 		else if (strstr(string,"SPECIAL OCCASION")) 	*Type = GSM_CAL_BIRTHDAY;
@@ -450,7 +457,7 @@ GSM_Error GSM_Calendar_GetValue(const GSM_CalendarEntry *note, int *start, const
  * Converts Gammu recurrence to vCal format. See GSM_DecodeVCAL_RRULE
  * for grammar description.
  */
-GSM_Error GSM_EncodeVCAL_RRULE(char *Buffer, int *Length, GSM_CalendarEntry *note, int TimePos, GSM_VCalendarVersion Version)
+GSM_Error GSM_EncodeVCAL_RRULE(char *Buffer, size_t *Length, GSM_CalendarEntry *note, int TimePos UNUSED, GSM_VCalendarVersion Version)
 {
 	int i;
 	int j;
@@ -677,11 +684,11 @@ GSM_Error GSM_EncodeVCAL_RRULE(char *Buffer, int *Length, GSM_CalendarEntry *not
 					if (Version == Mozilla_iCalendar) {
 						if (!header) {
 							*Length += sprintf(Buffer + (*Length), ";BYDAY=%s",
-								DaysOfWeek[note->Entries[j].Number]);
+								DaysOfWeek[note->Entries[i].Number]);
 							header = true;
 						} else {
 							*Length += sprintf(Buffer + (*Length), ",%s",
-								DaysOfWeek[note->Entries[j].Number]);
+								DaysOfWeek[note->Entries[i].Number]);
 						}
 					} else {
 						*Length += sprintf(Buffer + (*Length), " %s",
@@ -780,6 +787,7 @@ void GSM_ToDo_AdjustDate(GSM_ToDoEntry *note, GSM_DeltaTime *delta)
 			case TODO_ALARM_DATETIME :
 			case TODO_SILENT_ALARM_DATETIME:
 			case TODO_LAST_MODIFIED:
+			case TODO_START_DATETIME:
 				note->Entries[i].Date = GSM_AddTime(note->Entries[i].Date, *delta);
 				break;
 			case TODO_TEXT:
@@ -797,12 +805,12 @@ void GSM_ToDo_AdjustDate(GSM_ToDoEntry *note, GSM_DeltaTime *delta)
 	}
 }
 
-GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note, bool header, GSM_VCalendarVersion Version)
+GSM_Error GSM_EncodeVCALENDAR(char *Buffer, size_t *Length, GSM_CalendarEntry *note, bool header, GSM_VCalendarVersion Version)
 {
 	GSM_DateTime 	deltatime;
 	char 		dtstr[20];
 	char		category[100];
-	int		i, alarm = -1, date = -1;
+	int		i, alarm_pos = -1, date_pos = -1;
 
 	/* Write header */
 	if (header) {
@@ -825,7 +833,7 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note
 	for (i=0; i < note->EntriesNum; i++) {
 		switch (note->Entries[i].EntryType) {
 			case CAL_START_DATETIME :
-				date = i;
+				date_pos = i;
 				if (Version == Mozilla_iCalendar && (note->Type == GSM_CAL_MEMO || note->Type == GSM_CAL_BIRTHDAY)) {
 					SaveVCALDate(Buffer, Length, &note->Entries[i].Date, "DTSTART;VALUE=DATE");
 				} else {
@@ -840,14 +848,14 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note
 				}
 				break;
 			case CAL_TONE_ALARM_DATETIME :
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "AALARM");
 				}
 				break;
 			case CAL_SILENT_ALARM_DATETIME:
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "DALARM");
@@ -910,12 +918,12 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note
 			*Length+=sprintf(Buffer+(*Length), "RRULE:YM1%c%c",13,10);
 		}
 	} else {
-		GSM_EncodeVCAL_RRULE(Buffer, Length, note, date, Version);
+		GSM_EncodeVCAL_RRULE(Buffer, Length, note, date_pos, Version);
 	}
 
 	/* Include mozilla specific alarm encoding */
-	if (Version == Mozilla_iCalendar && alarm != -1 && date != -1) {
-		deltatime = VCALTimeDiff(&note->Entries[alarm].Date, &note->Entries[date].Date);
+	if (Version == Mozilla_iCalendar && alarm_pos != -1 && date_pos != -1) {
+		deltatime = VCALTimeDiff(&note->Entries[alarm_pos].Date, &note->Entries[date_pos].Date);
 
 		dtstr[0]='\0';
 		if (deltatime.Minute !=0) {
@@ -981,12 +989,10 @@ void GSM_ToDoFindDefaultTextTimeAlarmCompleted(GSM_ToDoEntry *entry, int *Text,
 	}
 }
 
-GSM_Error GSM_EncodeVTODO(char *Buffer, int *Length, GSM_ToDoEntry *note, bool header, GSM_VToDoVersion Version)
+GSM_Error GSM_EncodeVTODO(char *Buffer, size_t *Length, GSM_ToDoEntry *note, bool header, GSM_VToDoVersion Version)
 {
-	GSM_DateTime 	deltatime;
-	char 		dtstr[20];
 	char		category[100];
-	int		i, alarm = -1, date = -1;
+	int		i, alarm_pos = -1;
 
 	/* Write header */
 	if (header) {
@@ -1033,19 +1039,22 @@ GSM_Error GSM_EncodeVTODO(char *Buffer, int *Length, GSM_ToDoEntry *note, bool h
 				}
 				break;
 			case TODO_ALARM_DATETIME :
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "AALARM");
 				}
 				break;
 			case TODO_SILENT_ALARM_DATETIME:
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "DALARM");
 				}
 				break;
+			case TODO_START_DATETIME:
+				SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "DTSTART");
+				break;
 			case TODO_LAST_MODIFIED:
 				SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "LAST-MODIFIED");
 				break;
@@ -1085,35 +1094,6 @@ GSM_Error GSM_EncodeVTODO(char *Buffer, int *Length, GSM_ToDoEntry *note, bool h
 				/* Not supported */
 				break;
 		}
-
-		/* Include mozilla specific alarm encoding */
-		if (Version == Mozilla_iCalendar && alarm != -1 && date != -1) {
-			deltatime = VCALTimeDiff(&note->Entries[alarm].Date, &note->Entries[date].Date);
-
-			dtstr[0]='\0';
-			if (deltatime.Minute !=0) {
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-UNITS:minutes%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-LENGTH:%i%c%c",
-					deltatime.Minute,13,10);
-				sprintf(dtstr,"-PT%iM",deltatime.Minute);
-			} else if (deltatime.Hour !=0) {
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-UNITS:hours%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-LENGTH:%i%c%c",
-					deltatime.Hour,13,10);
-				sprintf(dtstr,"-PT%iH",deltatime.Hour);
-			} else if (deltatime.Day !=0) {
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-UNITS:days%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-LENGTH:%i%c%c",
-					deltatime.Day,13,10);
-				sprintf(dtstr,"-P%iD",deltatime.Day);
-			}
-			if (dtstr[0] != '\0') {
-				*Length+=sprintf(Buffer+(*Length), "BEGIN:VALARM%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "TRIGGER;VALUE=DURATION%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), " :%s%c%c",dtstr,13,10);
-				*Length+=sprintf(Buffer+(*Length), "END:VALARM%c%c",13,10);
-			}
-		}
 	}
 
 	*Length+=sprintf(Buffer+(*Length), "END:VTODO%c%c",13,10);
@@ -1594,14 +1574,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 	unsigned char 	Line[2000],Buff[2000];
 	int		Level = 0;
 	GSM_DateTime	Date;
-	GSM_DeltaTime	OneHour = {
-		.Timezone = 0,
-		.Second = 0,
-		.Minute = 0,
-		.Hour = 1,
-		.Day = 0,
-		.Month = 0,
-		.Year = 0};
+	GSM_DeltaTime	OneHour = {0, 0, 0, 1, 0, 0, 0};
 	GSM_TimeUnit	unit = GSM_TimeUnit_Unknown;
 	GSM_DeltaTime	trigger;
 	GSM_Error	error;
@@ -1621,7 +1594,6 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 	trigger.Timezone = -999;
 
 	if (CalVer == Mozilla_iCalendar && *Pos ==0) {
-		int error;
 		error = GSM_Make_VCAL_Lines (Buffer, &lBuffer);
 		if (error != ERR_NONE) return error;
 	}
@@ -1633,7 +1605,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 		switch (Level) {
 		case 0:
 			if (strstr(Line,"BEGIN:VEVENT")) {
-				Calendar->Type = -1;
+				Calendar->Type = 0;
 				date_only = true;
 				dstflag = 0;
 				Text=-1; Time=-1; Alarm=-1; EndTime=-1; Location=-1;
@@ -1641,6 +1613,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			}
 			if (strstr(Line,"BEGIN:VTODO")) {
 				ToDo->Priority 	= GSM_Priority_None;
+				ToDo->Type = GSM_CAL_MEMO;
 				dstflag = 0;
 				Text=-1; Time=-1; Alarm=-1; EndTime=-1; Location=-1;
 				Level 		= 2;
@@ -1648,9 +1621,17 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			break;
 		case 1: /* Calendar note */
 			if (strstr(Line,"END:VEVENT")) {
-				if (Time == -1) return ERR_UNKNOWN;
+				if (Time == -1) {
+					dbgprintf("vCalendar without date!\n");
+					return ERR_UNKNOWN;
+				}
 				if (rrule != NULL) {
-					error = GSM_DecodeVCAL_RRULE(rrule, Calendar, Time);
+					if (CalVer == Mozilla_iCalendar) {
+						/* @todo: We don't have parser for this right now */
+						error = ERR_NONE;
+					} else {
+						error = GSM_DecodeVCAL_RRULE(rrule, Calendar, Time);
+					}
 					free(rrule);
 					if (error != ERR_NONE) {
 						return error;
@@ -1680,7 +1661,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 
 				/* If event type is undefined choose appropriate type. Memos carry dates only, no times.
 				   Use Meetings for events with full date+time settings. */
-				if (Calendar->Type == -1) {
+				if (Calendar->Type == 0) {
 					if (date_only)
 						Calendar->Type = GSM_CAL_MEMO;
 					else
@@ -1694,7 +1675,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			   can defined in several ways. We will use the trigger value only since this is the value
 			   Mozilla calendar uses when importing ics-files. */
 			if (strncmp(Line, "UID:", 4) == 0) {
-				ReadVCALText(Line, "UID", Buff, CalVer == Mozilla_iCalendar);  // Any use for UIDs?
+				ReadVCALText(Line, "UID", Buff, CalVer == Mozilla_iCalendar);  /*  Any use for UIDs? */
 				break;
 			}
 			if (strstr(Line,"X-MOZILLA-ALARM-DEFAULT-UNITS:")) {
@@ -1837,7 +1818,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			}
 
 			if (strncmp(Line, "UID:", 4) == 0) {
-				ReadVCALText(Line, "UID", Buff, ToDoVer == Mozilla_iCalendar);  // Any use for UIDs?
+				ReadVCALText(Line, "UID", Buff, ToDoVer == Mozilla_iCalendar);  /*  Any use for UIDs? */
 				break;
 			}
 			if (strstr(Line,"X-MOZILLA-ALARM-DEFAULT-UNITS:")) {
@@ -1869,6 +1850,11 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 					ToDo->EntriesNum++;
 				}
 			}
+			if (ReadVCALDate(Line, "DTSTART", &Date, &is_date_only)) {
+				ToDo->Entries[ToDo->EntriesNum].Date = Date;
+				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_START_DATETIME;
+				ToDo->EntriesNum++;
+			}
 			if (ReadVCALDate(Line, "DALARM", &Date, &is_date_only)) {
 				ToDo->Entries[ToDo->EntriesNum].Date = Date;
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_SILENT_ALARM_DATETIME;
@@ -1942,7 +1928,37 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 	return ERR_NONE;
 }
 
-GSM_Error GSM_EncodeVNTFile(unsigned char *Buffer, int *Length, GSM_NoteEntry *Note)
+GSM_Error GSM_DecodeVNOTE(unsigned char *Buffer, int *Pos, GSM_NoteEntry *Note)
+{
+	unsigned char   Line[2000],Buff[2000];
+	int	     Level = 0;
+
+	Note->Text[0] = 0;
+	Note->Text[1] = 0;
+
+	while (1) {
+		MyGetLine(Buffer, Pos, Line, strlen(Buffer), true);
+		if (strlen(Line) == 0) break;
+		switch (Level) {
+		case 0:
+			if (strstr(Line,"BEGIN:VNOTE")) Level = 1;
+			break;
+		case 1:
+			if (strstr(Line,"END:VNOTE")) {
+				if (UnicodeLength(Note->Text) == 0) return ERR_EMPTY;
+				return ERR_NONE;
+			}
+			if (ReadVCALText(Line, "BODY",	      Buff, false)) {
+				CopyUnicodeString(Note->Text, Buff);
+			}
+			break;
+		}
+	}
+
+	return ERR_BUG;
+}
+
+GSM_Error GSM_EncodeVNTFile(unsigned char *Buffer, size_t *Length, GSM_NoteEntry *Note)
 {
 	*Length+=sprintf(Buffer+(*Length), "BEGIN:VNOTE%c%c",13,10);
 	*Length+=sprintf(Buffer+(*Length), "VERSION:1.1%c%c",13,10);
